#include "MagicianLibrary.bas"
// note: library must be in same directory unless you provide a path
main:
  gosub Initialization
  repeat
    ClearScr
    gosub MoveToStart 
    delay 4000 // give user time to present new card
    gosub FindCard
    if CardFound
      gosub RotateToNearVertical
      gosub RotateToVertical
      gosub CenterArmOnCard  
      gosub MoveCardToStack
    endif    
  until not CardFound
end  

  
  
RotateToNearVertical:
  print "Rotating card to near vertical"
  // measures vertical distance at current position
  // returns when dist. indicates the card is CLOSE to vertical
  // if card is nearly horizontal, card is rotated 90 degrees
  // otherwise, card is rotated slightly & arm is centered vertically
  // all operations are repeated until card is APPROXIMATELY vertical 
  NearVertical = false
  repeat
    gosub MeasureVerticalAndCenter
    if VertDist<28
      //card is nearly horiz, rotate 90 degrees
      call RotateCard(x,y,z,90) 
    elseif VertDist>47
      // card should be within 20 deg or so of vertical
      NearVertical = true
    else  
      // card is not vertical or horizontal, rotate 25 deg
      call RotateCard(x,y,z,25) 
    endif n
  until NearVertical
return

Sub RotateCard(x,y,z,deg)
  gosub PickupCard   
  call MoveTo(_QUICK,x,y,z,deg)
  call MoveTo(_QUICK,x,y,z-3,deg)
  call SuctionControl(_Drop)
  call MoveTo(_QUICK,x,y,z,deg)
  delay 100  
  call MoveTo(_QUICK,x,y,z,0)
return                 

PickupCard:
  call MoveTo(_LINEAR,x,y,z-3,0)
  call SuctionControl(_PickUp)
  call MoveTo(_LINEAR,x,y,z,0)
  delay 100
return

DropCard:
  call MoveTo(_LINEAR,x,y,z-3,0)
  call SuctionControl(_Drop)
  call MoveTo(_LINEAR,x,y,z,0)
  delay 100
return

RotateToVertical:
  print "Rotating card to vertical"
  // now that card is NEAR vertical
  // sensors are moved to edge of card until one or both fails to see card
  // if both sensors fail simultaneously, card is vertical
  // depending on which sensor failed, card is rotated toward vertical
  // all operations are repeated until card is vertical
  angle = 16
  repeat   // move toward edge
    y=y-1
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)
    if (front+back)=1                      // perhaps set angle based on additional moves needed
      // card is not yet straight - rotate
      if not front then a = -angle
      if not back then a = angle 
      y=y+inc
      call MoveTo(LINEAR,x,y,z,0) // move away from edge
      call RotateCard(x,y,z,a)
      angle = angle/2  // demonstrate simple successive approximation
      if angle<2 then angle=2
    endif        
  until (front+back)=0          
  // card now vertical - move back onto card
  y=y+inc
  call MoveTo(LINEAR,x,y,z,0)      
return

CenterArmOnCard:
  print "Centering arm on card"
  // card is assumed to be vertical
  y = y + inc
  call MoveTo(LINEAR,x,y,z,0) // move away from edge
  gosub MeasureVerticalAndCenter
  gosub CenterHorizontally
return

MeasureVerticalAndCenter:
  // sets VertDist to a number indicative of Vertical distance
  // information obtained is used to center arm vertically on card 
  // both sensors are assumed to initially be on card  
  Old = x // save current position
  // then move "down" until bottom sensor is off card
  repeat
    x=x-1
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)
  until back=0
  temp = x // lower position
  x = Old
  call MoveTo(LINEAR,x,y,z,0) // back to starting position
  repeat
    x=x+1
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)
  until front=0 
  VertDist = x-temp
  x = (x+temp)/2 // center position
  // center arm on the card vertically
  call MoveTo(LINEAR,x,y,z,0)  
return

CenterHorizontally:    
  // sets HorzDist to a number indicative of Horizontal distance
  // information obtained is used to center arm horizontally on card  
  // both sensors are assumed to be initially on card (vertically) 
  // then move "left" until back sensor is off card
  Old = y // remember starting position
  repeat
    y=y-1 
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)
  until not(front and back)          
  temp = y // lower position
  y = Old
  call MoveTo(LINEAR,x,y,z,0) // back to starting position
  repeat
    y=y+1
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)  
  until not(front and back)
  y = (y+temp)/2 // center position
  call MoveTo(LINEAR,x,y,z,0) 
return

MoveCardToStack:
  print "Moving card to stack"
  // it is assume arm is centered on a vertical card
  // card is moved to the stack location
  gosub PickupCard
  x = 215
  y = 150
  call MoveTo(JUMP,x,y,z,0)
  gosub DropCard   
return
  
FindCard:
  print "Looking for card"
  // finds card and gets both sensors on it
  for i=1 to 5
    y+=inc
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)
    if front or back then break
  next
  if not(front or back)
    print "Unable to find card"
    print
    print "Goodbye"\end
  endif
  if (front or back)
    // move a little forward (hopefully more toward center)
    y=y+(2*inc)
    call MoveTo(LINEAR,x,y,z,0)
    call ReadSensors(front,back)
    if not(front or back)
      // move back
      y-=inc
      call MoveTo(LINEAR,x,y,z,0)
    endif 
  if front and back
    CardFound = true
    return
  else
    // one sensor only, currently on card
    CardFound = false
    // decide on direction of future moves to find both sensors   
    if front then dir = 1          // front sensor over
    if back then dir = -1          // back sensor over
    for i=1 to 4    
      x=x+(dir*(inc/2)) 
      call MoveTo(LINEAR,x,y,z,0)       
      call ReadSensors(front,back)
      if front and back then CardFound = true\break
      if not(front or back) then break     
    next
return


Initialization:
  gosub InitArm
  delay 100
  // init ports
  call SetMultiplex(7,DigIn)
  call SetMultiplex(5,DigIn)
  SetColor Yellow,Blue
  inc = 15 // general movement size when finding card 
  print "                                                              "
  print "               Card Stacking Demonstration                    "
  print "                   by John Blankenship                        "
  print " ------------------------------------------------------------ "
  print " Two sensors are used to properly orient the card's position. "
  print " Each card presented will be stacked in a pile.               "
  print "                                                              "
  print " Search YouTube for 'Dobot Magicial Card Manipulation' to see "
  print " this program in action. Also see 'Dobot Magician Card Trick' " 
  print "                                                              "
  print " This program is written in RobotBASIC.                       "
  print " Download your free copy of RobotBASIC from RobotBASIC.org    "
  print "                                                              "
  SetColor Black,White
  print
  print
  WaitKey "Do you wish to HOME the arm? (y/n)",a 
  if (a=ascii("Y")) or (a=ascii("y")) then call Home()
return  

MoveToStart:
  print "Moving to starting position"
  // raise arm                        
  call MoveTo(QUICK,200,0,-20,0)
  delay 500 
  x=244
  y=-60
  z=-61
  call MoveTo(LINEAR,x,y,z,0)
return

sub ReadSensors(&a1,&a2)
  call DigitalInput(7,a1)
  call DigitalInput(5,a2)
  a1=not a1
  a2=not a2
return






