// Magician Library for Dobot Arm
//  written by John Blankenship
//   for use with RobotBASIC
//     available free from 
//     www.RobotBASIC.org
// library maybe freely used and
//   modified as long as these
//    comments are included

InitArm:
  dim b[4]
  SetCommPort 5,12 // 5 is the port number
  InlineInputMode ON
  // move modes
  JUMP = 0
  QUICK = 1
  LINEAR = 2
  ANGLES = 3 // take care not to exceed limits with this command  
  // suction control
  MotorOff = 0
  MotorON = 1
  PickUp = 1
  Drop = 0
  // I/O multiplex control
  NoConfig = 0
  PWM =1
  DigOut = 2
  DigIn = 3
  ADC = 4
return   

Sub Home() 
  ClearSerBuffer
  // forces arm to perform a physical home calibration
  SerialOut 0xAA,0xAA,3,31,3,1,0xDD 
  delay 1000  
  Serin RET
  // wait till arm finishes the home proceedure
  // only a 32 bit index is implemented here
  index = ascii(substring(RET,9,1))*(256^3)+ascii(substring(RET,8,1))*65536+ascii(substring(RET,7,1))*256 + ascii(substring(RET,6,1))
  repeat
    SerialOut 0xAA,0xAA,2,246,0,-246 // get current index to see if command has finished
    repeat  // must delay because arm does not send back anything during calibration with accelerometers
      CheckSerBuffer n
    until n=14     
    Serin RET    
    CurIndex =ascii(substring(RET,9,1))*(256^3)+ascii(substring(RET,8,1))*65536+ascii(substring(RET,7,1))*256 + ascii(substring(RET,6,1))
  until CurIndex>=index 
  delay 1000
return       

Sub SuctionControl(state)
  SerialOut 0xAA,0xAA,4,62,3,state,state
  SerialOut -(62+3+state+state) // checksum
  delay 500
  if state=0 then delay 500
  ClearSerBuffer
  delay 100
return  

Sub GripperControl(Enable,Grip)
  SerialOut 0xAA,0xAA,4,63,3,Enable,Grip
  SerialOut -(63+3+Enable+Grip)
  delay 500
  ClearSerialBuffer
return 
 
Sub GetPose(&x,&y,&z,&r)  
  delay 10
  ClearSerBuffer
  SerialOut 0xAA,0xAA,2,10,0,0xF6
  delay 10
  SerIn RET
  //delay d  // allow time for arm to stop moving
  b[0]=bin(ascii(SubString(RET,9,1)))
  b[1]=bin(ascii(SubString(RET,8,1)))
  b[2]=bin(ascii(SubString(RET,7,1)))
  b[3]=bin(ascii(SubString(RET,6,1)))
  s=""
  for i=0 to 3
    while Length(b[i])<8
      b[i]="0"+b[i]
    wend
    s=s+b[i]
  next 

  call ToFloat(s,x)

  b[0]=bin(ascii(SubString(RET,13,1)))
  b[1]=bin(ascii(SubString(RET,12,1))) 
  b[2]=bin(ascii(SubString(RET,11,1)))
  b[3]=bin(ascii(SubString(RET,10,1)))
  s=""
  for i=0 to 3
    while Length(b[i])<8
      b[i]="0"+b[i]
    wend
    s=s+b[i]
  next  
  call ToFloat(s,y)
  
  b[0]=bin(ascii(SubString(RET,17,1)))
  b[1]=bin(ascii(SubString(RET,16,1))) 
  b[2]=bin(ascii(SubString(RET,15,1)))
  b[3]=bin(ascii(SubString(RET,14,1)))
  s=""
  for i=0 to 3
    while Length(b[i])<8
      b[i]="0"+b[i]
    wend
    s=s+b[i]
  next 
  call ToFloat(s,z)
  
  b[0]=bin(ascii(SubString(RET,21,1)))
  b[1]=bin(ascii(SubString(RET,20,1))) 
  b[2]=bin(ascii(SubString(RET,19,1)))
  b[3]=bin(ascii(SubString(RET,18,1)))
  s=""
  for i=0 to 3
    while Length(b[i])<8
      b[i]="0"+b[i]
    wend
    s=s+b[i] 
  next 
  call ToFloat(s,r)
return  
  
Sub ToFloat(s,&v)
  if SubString(s,1,8)="00000000"
    v=0
    return
  endif
  ns=1 // number sign
  if SubString(s,1,1)="1" then ns=-1
  expon = (BinToInt("0%"+SubString(s,2,8)))-127
  // calc mantisa
  m=0
  wt = .5
  v = 1.0
  for i= 10 to 32   
    if SubString(s,i,1)="1" then v+=wt
    wt/=2
  next
  v*=(2.0^expon)
  v*=ns
  v=round(v*100)/100.0  
return
   

Sub SetMultiplex(Address,Configuration)
  SerialOut 0xAA,0xAA,4,130,3,Address,Configuration
  SerialOut -(130+3+Address+Configuration) 
  delay 10 
  Serin RET // thow away
return

Sub DigitalInput(Port,&answer)
  ClearSerBuffer
  SerialOut 0xAA,0xAA,5,133,0,Port,0,0
  SerialOut -(133+Port)
  delay 15
  checkSerBuffer n
  SerIn RET
  answer = ascii(Substring(RET,n-2,1))
return  

Sub AnalogInput(Port,&answer)
  ClearSerBuffer
  SerialOut 0xAA,0xAA,5,134,0,Port,0,0
  SerialOut -(134+Port) 
  delay 15 
  CheckSerBuffer n
  Serin RET                                        
  answer = ascii(Substring(RET,n-2,1))+ascii(Substring(RET,n-1,1))*256
return

Sub MoveTo(mode,x,y,z,r)
  ClearSerBuffer
  x=round(x*100)/100.0
  y=round(y*100)/100.0
  z=round(z*100)/100.0
  r=round(r*100)/100.0
  SerialOut 0xAA,0xAA,0x13,0x54,0x03,mode
  cksum=0x54+3+mode
  call ConvertToFloat(x)
  for i=0 to 3
    SerialOut b[i]
    cksum+=b[i]
  next
  call ConvertToFloat(y)
  for i=0 to 3
    SerialOut  b[i]
    cksum+=b[i]
  next
  call ConvertToFloat(z)
  for i=0 to 3
    SerialOut  b[i]
    cksum+=b[i]
  next
  call ConvertToFloat(r)
  for i=0 to 3
    SerialOut  b[i]
    cksum+=b[i]
  next  
  SerialOut -cksum
  delay 10
  repeat  // wait for return
    CheckSerBuffer n
  until n=14
  Serin RET 
  // only a 32 bit index is implemented here
  index = ascii(substring(RET,9,1))*(256^3)+ascii(substring(RET,8,1))*65536+ascii(substring(RET,7,1))*256 + ascii(substring(RET,6,1))
  for i= 6 to 13
  repeat
    SerialOut 0xAA,0xAA,2,246,0,-246 // get current index to see if command has finished
    repeat  // wait for return
      CheckSerBuffer n
    until n=14   
    Serin RET
    CurIndex =ascii(substring(RET,9,1))*(256^3)+ascii(substring(RET,8,1))*65536+ascii(substring(RET,7,1))*256 + ascii(substring(RET,6,1))
  until CurIndex>=index
return
 
test:
  for i=1 to Length(RET)
    print hex(ascii(SubString(RET,i,1)));
  next
return

 
Sub ConvertToFloat(n) //,&b1,&b2,&b3,&b4) 
  // simplified conversion - only works for for 2 sig. digital digits, .01 (okay most situations)
  mant = bin(abs(n*256)) //bin(abs(n))
  L = Length(mant)
//  if mant = "0" then L=0
  expo = bin(127-9+L) //bin(L-1+127) 
  // ensure expon is 8 bits
while Length(expo)<8
  expo="0"+expo
wend
if Length(expo)>8 then expo = Right(expo,8)
  if n=0
    expo = "00000000" 
    mant = "00000000"
  else
    // remove leading 1 (if it exists)
    mant = Right(mant,Length(mant)-1)   
  endif 
  if n>=0  
    bytes = "0"+expo+mant
  else
    bytes = "1"+expo+mant
  endif
  while length(bytes)<32
    bytes = bytes+"0"
  wend
  // separate binary into bytes
  b[3] = SubString(bytes,1,8)
  b[2] = SubString(bytes,9,8)
  b[1] = SubString(bytes,17,8)
  b[0] = SubString(bytes,25,8)  
  // and convert to decimal  
  for i=0 to 3
    val = 0
    w=1
    for p=8 to 1
      if SubString(b[i],p,1)="1" then val+=w
      w*=2
    next
    b[i]=val   
  next
return
